package org.gcube.portlets.user.geoportaldataentry.client;

import java.util.List;

import org.gcube.application.geoportalcommon.shared.GeoNaItemRef;
import org.gcube.application.geoportalcommon.shared.ResultSetPaginatedData;
import org.gcube.application.geoportalcommon.shared.SearchingFilter;
import org.gcube.application.geoportalcommon.shared.geoportal.ResultDocumentDV;
import org.gcube.application.geoportalcommon.shared.geoportal.config.ActionDefinitionDV;
import org.gcube.application.geoportalcommon.shared.geoportal.project.LifecycleInformationDV;
import org.gcube.application.geoportalcommon.shared.geoportal.project.ProjectDV;
import org.gcube.application.geoportalcommon.shared.geoportal.project.RelationshipDV;
import org.gcube.application.geoportalcommon.shared.geoportal.ucd.UseCaseDescriptorDV;
import org.gcube.application.geoportalcommon.shared.products.ConcessioneDV;
import org.gcube.application.geoportalcommon.shared.products.content.WorkspaceContentDV;
import org.gcube.application.geoportalcommon.shared.products.paths.FileSetPathsDV;
import org.gcube.portlets.user.geoportaldataentry.shared.CommitReport;
import org.gcube.portlets.user.geoportaldataentry.shared.GNADataEntryExtendedConfigProfile;
import org.gcube.portlets.user.geoportaldataentry.shared.GeoNaFormDataObject;
import org.gcube.portlets.user.geoportaldataentry.shared.GeoportalISConfig;
import org.gcube.portlets.user.geoportaldataentry.shared.Tree_Node;
import org.gcube.portlets.widgets.mpformbuilder.shared.GenericDatasetBean;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * The Interface GeoportalDataEntryServiceAsync.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 *         Sep 1, 2022
 */
public interface GeoportalDataEntryServiceAsync {

	/**
	 * Utility class to get the RPC Async interface from client-side code
	 */
	public static final class Util {
		private static GeoportalDataEntryServiceAsync instance;

		public static final GeoportalDataEntryServiceAsync getInstance() {
			if (instance == null) {
				instance = (GeoportalDataEntryServiceAsync) GWT.create(GeoportalDataEntryService.class);
			}
			return instance;
		}

		private Util() {
			// Utility class should not be instantiated
		}
	}

	void saveGeonaDataForms(String profileID, Tree_Node<GeoNaFormDataObject> tree_Node,
			AsyncCallback<CommitReport> callback);

	void getGeonaInitConfig(AsyncCallback<GeoportalISConfig> callback);

	void getLinksFor(String itemId, String profileID, AsyncCallback<GeoNaItemRef> callback);

	void getListProjects(String theProfileID, Integer start, Integer limit, SearchingFilter filter,
			boolean reloadFromService, AsyncCallback<ResultSetPaginatedData> callback);

	void deleteProject(String profileID, String projectID, AsyncCallback<Boolean> callback);

	void getJSONDocumentInTheProject(String profileID, String projectID, AsyncCallback<String> callback);

	void updateRecord(String profileID, String projectID, String jsonUpdate, AsyncCallback<ProjectDV> callback);

	void readFileSetPaths(AsyncCallback<FileSetPathsDV> callback);

	void updateSectionForRecord(String itemId, String recordType, String section, int pathIndex,
			List<WorkspaceContentDV> keepCurrentContent, GenericDatasetBean gDBean,
			AsyncCallback<ConcessioneDV> callback);

	void readDataViewerConfig(AsyncCallback<GNADataEntryExtendedConfigProfile> asyncCallback);

	void getListUseCaseDescriptors(List<String> handlersIds, AsyncCallback<List<UseCaseDescriptorDV>> callback);

	void getLifecycleInfoForProjectId(String profileID, String projectID,
			AsyncCallback<LifecycleInformationDV> callback);

	void performActionSteps(String profileID, String projectID, ActionDefinitionDV action,
			AsyncCallback<ProjectDV> callback);

	void getRelationshipNames(String profileID, AsyncCallback<List<String>> callback);

	void createRelationship(String fromProfileID, String fromProjectID, String relationshipName, String toProfileID,
			String toProjectID, AsyncCallback<RelationshipDV> callback);

	void getProjectByID(String profileID, String projectID, AsyncCallback<ProjectDV> callback);

	void getResultDocumentFoProjectByID(String profileID, String projectID, AsyncCallback<ResultDocumentDV> callback);

}
